/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System;
using System.IO;
using System.Runtime.InteropServices;
using System.Runtime.CompilerServices;

namespace Tizen.Runtime
{
    internal static class ALog
    {
        const string Library = "libdlog.so.0";
        const string Tag = "DOTNET_LAUNCHER";

        internal enum LogPriority
        {
            DLOG_UNKNOWN = 0,
            DLOG_DEFAULT,
            DLOG_VERBOSE,
            DLOG_DEBUG,
            DLOG_INFO,
            DLOG_WARN,
            DLOG_ERROR,
            DLOG_FATAL,
            DLOG_SILENT,
            DLOG_PRIO_MAX,
        }

        public static void Debug(string message,
                [CallerFilePath] string file = "",
                [CallerMemberName] string function = "",
                [CallerLineNumber] int line = 0)
        {
            Print(LogPriority.DLOG_DEBUG, Tag, message, file, function, line);
        }

        public static void Info(string message,
                [CallerFilePath] string file = "",
                [CallerMemberName] string function = "",
                [CallerLineNumber] int line = 0)
        {
            Print(LogPriority.DLOG_INFO, Tag, message, file, function, line);
        }

        public static void Error(string message,
                [CallerFilePath] string file = "",
                [CallerMemberName] string function = "",
                [CallerLineNumber] int line = 0)
        {
            Print(LogPriority.DLOG_ERROR, Tag, message, file, function, line);
        }

        [DllImportAttribute(Library, EntryPoint = "dlog_print")]
        internal static extern int Print(LogPriority priority, string tag, string format, string file, string function, int line, string msg);

        private static void Print(LogPriority priority, string tag, string message, string file, string function, int line)
        {
            FileInfo fileInfo = new FileInfo(file);
            Print(priority, tag, "%s: %s(%d) > %s", fileInfo.Name, function, line, message);
        }

    }
}
